package repository

import (
	"ai-gateway/common/utils"
	"code.mylomen.com/syj/lib/context"
	"time"
)

// FThirdCorpUserAccountDO 第三方 企业用户 帐号表
type FThirdCorpUserAccountDO struct {
	Id         uint64    `gorm:"column:id;type:bigint(20) unsigned;primary_key;AUTO_INCREMENT" json:"id"`
	Sn         string    `gorm:"column:sn;type:varchar(64);comment:sn" json:"sn"`
	CorpId     uint64    `gorm:"column:corp_id;type:bigint(20) unsigned;comment:企业id;NOT NULL" json:"corp_id"`
	BizId      int64     `gorm:"column:biz_id;type:bigint(20);comment:业务id。 " json:"biz_id"`
	Name       string    `gorm:"column:name;type:varchar(64);comment:名称" json:"name"`
	Admin      uint      `gorm:"column:admin;type:tinyint(1) unsigned;default:0;comment:是否管理员。0:否；1:是;NOT NULL" json:"admin"`
	Account    string    `gorm:"column:account;type:varchar(64);comment:帐号" json:"account"`
	Pwd        string    `gorm:"column:pwd;type:varchar(128);comment:密码" json:"pwd"`
	Status     int       `gorm:"column:status;type:tinyint(1) unsigned;default:1;comment:是否有效。0:否；1:是;NOT NULL" json:"status"`
	Deleted    uint      `gorm:"column:deleted;type:tinyint(1) unsigned;default:0;comment:逻辑删除。 0:未删除；1:已删除;NOT NULL" json:"deleted"`
	CreateTime time.Time `gorm:"column:create_time;type:timestamp;default:CURRENT_TIMESTAMP;comment:创建时间;NOT NULL" json:"create_time"`
	UpdateTime time.Time `gorm:"column:update_time;type:timestamp;default:CURRENT_TIMESTAMP;comment:更新时间;NOT NULL" json:"update_time"`
}

func (m *FThirdCorpUserAccountDO) TableName() string {
	return "f_third_corp_user_account"
}

type thirdCorpUserAccountRepositoryImpl struct {
}

func (thirdCorpUserAccountRepositoryImpl) QueryPage(ctx context.GormWithZap, page, size int) *[]FThirdCorpUserAccountDO {
	db, err := ctx.GormWithContext()
	if err != nil {
		return nil
	}

	offset := (page - 1) * size
	offset = utils.If(offset >= 0, offset, 0)
	var dataList []FThirdCorpUserAccountDO
	sqlErr := db.Model(&FThirdCorpUserAccountDO{}).Order("id DESC").Limit(size).Offset(offset).Find(&dataList).Error
	if sqlErr != nil {
		ctx.Zap().Sugar().Errorf("QueryPage_sqlErr page:%d %+v", page, sqlErr)
		return nil
	}

	if len(dataList) == 0 {
		return nil
	}

	return &dataList

}

func (thirdCorpUserAccountRepositoryImpl) FindById(ctx context.GormWithZap, id uint64) *FThirdCorpUserAccountDO {
	db, err := ctx.GormWithContext()
	if err != nil {
		return nil
	}

	var model FThirdCorpUserAccountDO
	sqlErr := db.Where("id=?", id).First(&model).Error
	if sqlErr != nil {
		ctx.Zap().Sugar().Errorf("FindById_sqlErr id:%d %+v", id, sqlErr)
		return nil
	}

	return &model
}

func (thirdCorpUserAccountRepositoryImpl) Count(ctx context.GormWithZap) int64 {
	db, err := ctx.GormWithContext()
	if err != nil {
		return 0
	}

	var total int64
	sqlErr := db.Model(&FThirdCorpUserAccountDO{}).Select("count(*)").First(&total).Error
	if sqlErr != nil {
		ctx.Zap().Sugar().Errorf("Count_sqlErr  %+v", sqlErr)
		return 0
	}

	return total
}

func (thirdCorpUserAccountRepositoryImpl) Create(ctx context.GormWithZap, data *FThirdCorpUserAccountDO) error {
	db, err := ctx.GormWithContext()
	if err != nil {
		return err
	}

	//保存数据
	return db.Create(data).Error
}

func (thirdCorpUserAccountRepositoryImpl) UpdateById(ctx context.GormWithZap, data FThirdCorpUserAccountDO) error {
	db, err := ctx.GormWithContext()
	if err != nil {
		return err
	}

	//更新数据
	return db.Updates(&data).Error

}

func (thirdCorpUserAccountRepositoryImpl) UpdateStatusById(ctx context.GormWithZap, id uint64, status int, operator string) error {
	db, err := ctx.GormWithContext()
	if err != nil {
		return err
	}

	data := FThirdCorpUserAccountDO{}
	sqlErr := db.Model(&data).
		Select("status").
		Where("id=?", id).
		Updates(FThirdCorpUserAccountDO{Status: status}).Error
	if sqlErr != nil {
		return sqlErr
	}

	return nil
}

func (thirdCorpUserAccountRepositoryImpl) deleteById(ctx context.GormWithZap, id uint64, operator string) error {
	db, err := ctx.GormWithContext()
	if err != nil {
		return err
	}

	data := FThirdCorpUserAccountDO{}
	sqlErr := db.Model(&data).
		Select("deleted").
		Where("id=?", id).
		Updates(FThirdCorpUserAccountDO{Deleted: 1}).Error
	if sqlErr != nil {
		return sqlErr
	}

	return nil
}

func (thirdCorpUserAccountRepositoryImpl) FindThirdAccountBy(ctx context.GormWithZap, bizId, corpId uint64, account string) *FThirdCorpUserAccountDO {
	db, err := ctx.GormWithContext()
	if err != nil {
		return nil
	}

	var model FThirdCorpUserAccountDO
	sqlErr := db.Where("biz_id=? and corp_id=? And account=?", bizId, corpId, account).First(&model).Error
	if sqlErr != nil {
		ctx.Zap().Sugar().Errorf("FindThirdAccountBy_sqlErr corpId:%d account:%s %+v", corpId, account, sqlErr)
		return nil
	}

	return &model
}
